#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <stdlib.h>
#include <portbits.h>
#include "lc7981.h"
#include "6x8_horizontal_MSB_1.h"
#define F_CPU	8000000UL
#include <util/delay.h>

#define RS			PORTD_2
#define RW			PORTD_3
#define Enable		PORTD_4
#define CS			PORTD_6
#define RES			PORTD_7

#define LCD_DDR		DDRB
#define LCD_PORT	PORTB
#define LCD_PIN		PINB


#define COMDELAY	40			// min: 18/fclkLCD[MHz]

unsigned char lcd_xpos, lcd_ypos;

void lcd_init(void)
{	DDRD=252;
	LCD_DDR=255;

#ifdef RES
	RES=0;
	_delay_us(100);
	RES=1;
#endif
	_delay_ms(10);

#ifdef CS
	CS=0;
#endif
	lcd_writecom(0x01);			// character pitch
	lcd_writedat(0x77);			// 8x8 chars

	lcd_writecom(0x02);			// characters
	lcd_writedat(19);
	
	lcd_writecom(0x08);			// startadress low
	lcd_writedat(0x00);			// 0

	lcd_writecom(0x09);			// startadress high
	lcd_writedat(0x00);			// 0
	
	lcd_writecom(0x03);			// set duty
	lcd_writedat(79);			// 1/80

	lcd_writecom(0x00);			// mode con
	lcd_writedat(0x32);			// display on
#ifdef CS
	CS=1;
#endif

	lcd_clear();
}

void lcd_waitbusy(void)
{
#ifdef RW
	unsigned char temp;
#ifdef CS
	CS=0;
#endif
	RW=1;
	RS=1;
	LCD_DDR=0;
	do
	{	_delay_us(0.5);
		Enable=1;
		_delay_us(0.6);
		temp=LCD_PIN;
		Enable=0;
	}
	while (temp&128);
	LCD_DDR=255;
#ifdef CS
	CS=1;
#endif
#else
	_delay_us(COMDELAY);
#endif
}

void lcd_write(unsigned char byte)
{
#ifdef CS
	CS=0;
#endif
#ifdef RW
	RW=0;
#endif
	LCD_PORT=byte;
	_delay_us(0.1);
	Enable=1;
	_delay_us(0.8);
	Enable=0;
#ifdef CS
	CS=1;
#endif
}

void lcd_writedat(unsigned char byte)
{
	RS=0;
	lcd_write(byte);
}

void lcd_writecom(unsigned char byte)
{
	lcd_waitbusy();
	RS=1;
	lcd_write(byte);
}

void lcd_clear(void)
{	unsigned short i;
	lcd_setadress(0,0);
	for (i=0; i<160/8*80; i++)
		lcd_writebyte(0);

}

void lcd_setadress(unsigned char x, unsigned char y)
{	unsigned short adress;
	adress=y*20+x;
	lcd_writecom(0x0A);				// writeadress low
	lcd_writedat(adress&255);
	lcd_writecom(0x0B);				// writeadress high
	lcd_writedat(adress/256);
}

void lcd_writebyte(unsigned char byte)
{	lcd_writecom(0x0C);
	lcd_writedat(byte);
}

void lcd_fill(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char col)
{	unsigned char y,x;
	for (y=y1;y<=y2; y++)
	{	for (x=x1;x<=x2; x++)
		{	if (col)
				lcd_setpixel(x,y);
			else
				lcd_clrpixel(x,y);
		}
	}
}

void lcd_setpixel(unsigned char x, unsigned char y)
{	if ((x>=160)||(y>=80))
		return;
	lcd_setadress(x/8,y);
	lcd_writecom(0x0F);
	lcd_writedat(x&7);
}

void lcd_clrpixel(unsigned char x, unsigned char y)
{	if ((x>=160)||(y>=80))
		return;
	lcd_setadress(x/8,y);
	lcd_writecom(0x0E);
	lcd_writedat(x&7);
}

void lcd_gotoxy(unsigned char x, unsigned char y)
{	lcd_xpos=x;
	lcd_ypos=y;
}

void lcd_writechar(unsigned char b)
{	unsigned char i,j,c;
	if (lcd_xpos+5>=160)
	{	lcd_xpos=0;
		if (lcd_ypos+7>=80)
			lcd_ypos=0;
	}
	for (i=0; i<8; i++)
	{	c=pgm_read_byte(&font[b][i]);
		for (j=0; j<6; j++)
		{	if (c&(32>>j))
				lcd_setpixel(lcd_xpos+j,lcd_ypos+i);
			else
				lcd_clrpixel(lcd_xpos+j,lcd_ypos+i);
		}
	}
	lcd_xpos+=6;
}

void lcd_writestringP(const char *string)
{	char c;
	for(;;)
	{	c=pgm_read_byte(string++);
		if (!c)
			break;
		lcd_writechar(c);
	}
}

void bresenham_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2)
{	signed short dx,dy,x,y;
	signed char sdx,sdy;
	unsigned short i,px,py;
	dx=(signed int)x2-x1;      // the horizontal distance of the line
	dy=(signed int)y2-y1;      // the vertical distance of the line
	if (dx==0)
		sdx=0;
	if (dx>0)
		sdx=1;
	else
	{  	sdx=-1;
		dx=-dx;
	}
	if (dy==0)
		sdy=0;
	if (dy>0)
		sdy=1;
	else
	{	sdy=-1;
		dy=-dy;
	}
	x=dx*2;
	y=dy*2;
	px=x1;
	py=y1;
	lcd_setpixel (px,py);

	if (dx>=dy)					 // the line is more horizontal than vertical
	{	for(i=0;i<dx;i++)
		{	y+=dy;
			if (y>=dx)
			{	y-=dx;
				py+=sdy;
			}
			px+=sdx;
			lcd_setpixel (px,py);
		}
	}
	else				 		// the line is more vertical than horizontal
	{	for(i=0;i<dy;i++)
		{	x+=dx;
			if (x>=dy)
			{	x-=dy;
				px+=sdx;
			}
			py+=sdy;
			lcd_setpixel (px,py);
		}
	}
}

void lcd_line (unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{   unsigned char temp;
	if (y1==y2)
	{	if (x1>x2)
		{	temp=x1;
			x1=x2;
			x2=temp;
		}
		for (temp=x1; temp<=x2; temp++)
			lcd_setpixel (temp,y1);

	}
	else if (x1==x2)
	{	if (y1>y2)
		{	temp=y1;
			y1=y2;
			y2=temp;
		}
		for (temp=y1; temp<=y2; temp++)
			lcd_setpixel (x1,temp);
	}
	else
		bresenham_line (x1, y1, x2, y2);
}
